/*!\file window.cpp
 *
 * \brief Utilisation du flux vido OpenCV avec GL4Dummies et d'OpenGL 3+
 *
 * \author Fars BELHADJ, amsi@ai.univ-paris8.fr
 * \date November 06 2018
 */
#include <GL4D/gl4du.h>
#include <GL4D/gl4dg.h>
#include <GL4D/gl4duw_SDL2.h>
#include <opencv2/opencv.hpp>
#include <iostream>

using namespace cv;
using namespace std;

/* Prototypes des fonctions statiques contenues dans ce fichier C */
static void init(void);
static void resize(int w, int h);
static void draw(void);
static void quit(void);
/*!\brief dimensions de la fentre */
static int _w = 640, _h = 480;
/*!\brief identifiant du quadrilatre */
static GLuint _quad = 0;
/*!\brief identifiants des (futurs) GLSL programs */
static GLuint _pId = 0;
/*!\brief identifiant de la texture charge */
static GLuint _tId = 0;
/*!\brief device de capture OpenCV */
/* essayer d'ouvrir la seconde camra */
VideoCapture _cap(1);

/*!\brief Cration de la fentre et paramtrage des fonctions callback.*/
int main(int argc, char ** argv) {
  /* si echec */
  if(!_cap.isOpened()) {
    /* essayer d'ouvrir n'importe quelle camra */
    _cap.open(CV_CAP_ANY);
    /* si echec, quitte */
    if(!_cap.isOpened())
      return 1;
  }
  /* on modifie les dimensions de capture */
  _cap.set(CV_CAP_PROP_FRAME_WIDTH,  640);
  _cap.set(CV_CAP_PROP_FRAME_HEIGHT, 480);
  /* on rcupre et imprime les dimensions de la vido */
  Size s(_cap.get(CV_CAP_PROP_FRAME_WIDTH), _cap.get(CV_CAP_PROP_FRAME_HEIGHT));
  _w = (int)s.width;
  _h = (int)s.height;
  if(!gl4duwCreateWindow(argc, argv, "GL4Dummies", SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, 
			 _w, _h, SDL_WINDOW_RESIZABLE | SDL_WINDOW_SHOWN))
    return 1;
  init();
  atexit(quit);
  gl4duwResizeFunc(resize);
  gl4duwDisplayFunc(draw);
  gl4duwMainLoop();
  return 0;
}
/*!\brief Initialise les paramtres OpenGL.*/
static void init(void) {
  _pId = gl4duCreateProgram("<vs>shaders/basic.vs", "<fs>shaders/basic.fs", NULL);
  glClearColor(0.2f, 0.2f, 0.2f, 0.0f);
  resize(_w, _h);
  _quad = gl4dgGenQuadf();
  glGenTextures(1, &_tId);
  glBindTexture(GL_TEXTURE_2D, _tId);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 1, 1, 0, GL_BGRA, GL_UNSIGNED_BYTE, NULL);
}
/*!\brief Paramtrage de la vue (viewPort) OpenGL en fonction
 * des dimensions de la fentre.
 * \param w largeur de la fentre transmise par GL4Dummies.
 * \param h hauteur de la fentre transmise par GL4Dummies.
 */
static void resize(int w, int h) {
  _w  = w; _h = h;
  glViewport(0, 0, _w, _h);
}
/*!\brief Dessin de la gomtrie texture. */
static void draw(void) {
  Mat frame;
  /* rcuprer la frame courante */
  _cap >> frame;
  glBindTexture(GL_TEXTURE_2D, _tId);
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, frame.cols, frame.rows, 0, GL_BGR, GL_UNSIGNED_BYTE, frame.ptr());
  glUseProgram(_pId);
  glDisable(GL_DEPTH_TEST);
  glBindTexture(GL_TEXTURE_2D, _tId);
  glUniform1i(glGetUniformLocation(_pId, "inv"), 1);
  glUniform1i(glGetUniformLocation(_pId, "myTexture"), 0);
  gl4dgDraw(_quad);
}
/*!\brief Appele au moment de sortir du programme (atexit). Elle
 *  libre les donnes utilises par OpenGL et GL4Dummies.*/
static void quit(void) {
  if(_tId)
    glDeleteTextures(1, &_tId);
  gl4duClean(GL4DU_ALL);
}
